@@ -503,6 +503,14 @@ class CloudPods:
             log.error(f"Failed to request /servers: Error.\rerr msg is {str(e)}",exc_info=True)
             return False
 
+    def get_can_deploy_server_host_num(self,os_arch,bios_mode):
+        hosts=[]
+        for elem in self.get_baremetal_hosts():
+            if "user:test_os_type" in elem["metadata"].keys() and elem["metadata"]["user:test_os_type"]==f"{os_arch.lower()}_{bios_mode.lower()}":
+                if not self.host_has_baremetal_server(elem["id"]):
+                    hosts.append(elem)
+        return len(hosts)
+
     @enter_and_leave_function
     def get_host_spec(self,host_id):
         try:
@@ -526,27 +534,94 @@ class CloudPods:
             return None
 
     @enter_and_leave_function
-    def get_baremetal_hosts(self):
-        try:
-            get_all_baremetal_hosts = f"/hosts"
-            url = self.__compute_url + get_all_baremetal_hosts
-            payload = {
-                "scope": "system",
-                "details": True,
-                "baremetal": True,
-                "with_meta": True,
-                "summary_stats":  True
-            }
-            rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
+    def get_baremetal_hosts(self,page_size=10):
+        """
+            分页获取所有裸金属主机信息（自动循环获取全量数据）
+
+            Args:
+                page_size: 每页获取的数量（默认50，可根据API限制调整）
+
+            Returns:
+                list: 所有裸金属主机的完整数据列表（若失败返回空列表）
+            """
+        all_hosts = []  # 存储所有页的结果
+        page_num = 1  # 起始页码（从1开始，根据API实际规则调整）
+        url = f"{self.__compute_url}/hosts"  # 接口URL
+
+        # 基础请求参数（添加分页相关参数）
+        base_payload = {
+            "scope": "system",
+            "details": True,
+            "baremetal": True,
+            "with_meta": True,
+            "summary_stats": True,
+            "page_size": page_size  # 每页数量
+        }
+
+        log.info(f"开始分页获取裸金属主机信息，每页{page_size}条...")
+
+        while True:
             try:
-                rs_data = rs.json()
-                return rs_data
+                # 添加当前页码参数
+                payload = base_payload.copy()
+                payload["page_num"] = page_num  # 假设API用page_num表示页码（根据实际接口调整）
+
+                # 发送请求
+                rs = self.__session.get(
+                    url=url,
+                    params=payload,
+                    verify=False,
+                    timeout=600
+                )
+                rs.raise_for_status()  # 触发HTTP错误（如404、500）
+
+                # 解析响应
+                try:
+                    rs_data = rs.json()
+                except ValueError as e:
+                    log.error(
+                        f"第{page_num}页响应解析失败（非JSON格式），响应内容：{rs.text[:500]}...",
+                        exc_info=True
+                    )
+                    break  # 解析失败，终止分页（或根据需求重试）
+
+                # 验证返回数据结构（假设数据在rs_data["items"]中，根据实际API调整）
+                if not isinstance(rs_data, dict) or "hosts" not in rs_data:
+                    log.error(f"第{page_num}页响应格式异常，缺少'items'字段：{rs_data}")
+                    break
+
+                current_page_hosts = rs_data["hosts"]
+                all_hosts.extend(current_page_hosts)  # 合并当前页数据
+
+                # 日志：记录当前页进度
+                log.debug(
+                    f"已获取第{page_num}页，本页{len(current_page_hosts)}条，累计{len(all_hosts)}条"
+                )
+
+                # 判断是否还有下一页（根据API返回的分页信息，如total、page_num、page_size）
+                total = rs_data.get("total", 0)  # 总条数（假设API返回total字段）
+                if len(all_hosts) >= total:
+                    log.info(f"所有裸金属主机信息获取完成，共{len(all_hosts)}条")
+                    break
+
+                # 准备请求下一页
+                page_num += 1
+
+            except requests.exceptions.HTTPError as e:
+                log.error(
+                    f"第{page_num}页请求失败（HTTP错误），状态码：{rs.status_code}，响应：{rs.text[:500]}...",
+                    exc_info=True
+                )
+                # 非致命错误（如429限流）可考虑重试，此处简单终止
+                break
+            except requests.exceptions.RequestException as e:
+                log.error(f"第{page_num}页请求异常（网络/超时等）：{str(e)}", exc_info=True)
+                break
             except Exception as e:
-                log.warning(f"Failed to get baremetal hosts: Error. rs is {rs.text}.\rerr msg is {str(e)}",exc_info=True)
-                return None
-        except Exception as e:
-            log.warning(f"Failed to request /hosts: Error.\rerr msg is {str(e)}",exc_info=True)
-            return None
+                log.error(f"第{page_num}页处理未知错误：{str(e)}", exc_info=True)
+                break
+
+        return all_hosts
 
     @enter_and_leave_function
     def get_baremetal_servers(self):
@@ -616,10 +691,18 @@ if __name__ == '__main__':
     # print(server_ip)
     # cloudpods = CloudPods('https://10.240.30.110:30500/v3', 'admin', 'jSj@2008')
     cloudpods = CloudPods('https://10.20.40.101:30500/v3', 'admin', 'jSj@2008')
+    # rs=cloudpods.get_baremetal_hosts()
+    # for elem in rs:
+    #     if elem["id"]=="5d6dfb67-db25-43d2-8057-116769fb3abf":
+    #         print(elem)
+    rs = cloudpods.get_can_deploy_server_host_num("x86_64","uefi")
+    print(rs)
+    # print(len(rs))
+    # print(rs[0])
     # rs=cloudpods.get_server_status("b43ba471-471a-44c1-8753-a7b8a9ead05c")
     # print(rs)
-    rs=cloudpods.wait_for_server_is_deleted("fb4b6563-f43e-404b-8b74-394c331eb89b")
-    print(rs)
+    # rs=cloudpods.wait_for_server_is_deleted("fb4b6563-f43e-404b-8b74-394c331eb89b")
+    # print(rs)
     # rs=cloudpods.get_baremetal_hosts()
     # print(rs)
     # rs=cloudpods.get_server_detail("8c925c8e-752f-4358-8c3e-a80847732d91")
